package com.ccb.mobile.platform.http;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.util.CharArrayBuffer;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Document;

import com.ccb.mobile.platform.system.BaseSystem;
import com.ccb.mobile.platform.utils.MbsLogManager;

import android.util.Log;

public class Response {
	private final HttpResponse mResponse;
	private boolean mStreamConsumed = false;
	private String mAccept_Ranges = null;//断点续传，请求头Accept-Ranges
	private String mContent_Range = null;//断点续传，请求头Content-Range
	public Response(HttpResponse res) {
		mResponse = res;
	}

	/**
	 * Convert Response to inputStream
	 * 
	 * @return InputStream or null
	 * @throws ResponseException
	 */
	public InputStream asStream() throws ResponseException {
		try {
			final HttpEntity entity = mResponse.getEntity();
			if(MbsLogManager.IS_DEBUG){
				long contentLength = entity.getContentLength();
				MbsLogManager.logD("-----流长度："+contentLength);
			}
			if (entity != null) {
				return entity.getContent();
			}
		} catch (IllegalStateException e) {
			throw new ResponseException(e.getMessage(), e);
		} catch (IOException e) {
			throw new ResponseException(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * @deprecated use entity.getContent();
	 * @param entity
	 * @return
	 * @throws ResponseException
	 */
	@SuppressWarnings("unused")
	private InputStream asStream(HttpEntity entity) throws ResponseException {
		if (null == entity) {
			return null;
		}

		InputStream is = null;
		try {
			is = entity.getContent();
		} catch (IllegalStateException e) {
			throw new ResponseException(e.getMessage(), e);
		} catch (IOException e) {
			throw new ResponseException(e.getMessage(), e);
		}

		// mResponse = null;
		return is;
	}

	/**
	 * Convert Response to Context String
	 * 
	 * @return response context string or null
	 * @throws ResponseException
	 */
	public String asString(final String chartSet, boolean isCallISO_GBK) throws ResponseException {
		try {
			return Response.entityToString(mResponse.getEntity(),chartSet,isCallISO_GBK);
		} catch (IOException e) {
			throw new ResponseException(e.getMessage(), e);
		}
	}

	/**
	* This isResponse.java class 
	* @param entity  网络返回交易信息实体
	* @param character_code 请求时设置的编码，与交易配置表一致
	* @param isCallISO_GBK 交易已调用CallISO_GBK流程，该交易需要转码由ISO-8859-1转为GBK
	* @return 返回转码的结果
	* @throws IOException
	* @throws ResponseException
	* @return String: 返回类型
	 */
	public static String entityToString(final HttpEntity entity,final String chartSet,final boolean isCallISO_GBK) throws IOException,
			ResponseException {
		if (null == entity) {
			throw new IllegalArgumentException("HTTP entity may not be null");
		}
		InputStream instream = entity.getContent();
		// InputStream instream = asStream(entity);
		if (instream == null) {
			return "";
		}
		if (entity.getContentLength() > Integer.MAX_VALUE) {
			throw new IllegalArgumentException(
					"HTTP entity too large to be buffered in memory");
		}

		int i = (int) entity.getContentLength();
		if (i < 0) {
			i = 4096;
		}
		if(MbsLogManager.IS_DEBUG){
			Log.i("LDS", i + " content length"+"--encoding--"+chartSet);
		}
		Reader reader = null;
		//遗留交易，需要将iso-8859-1转成gbk
		if(isCallISO_GBK){
			reader = new BufferedReader(new InputStreamReader(instream, HttpClient.GBK));
		}else{
			reader = new BufferedReader(new InputStreamReader(instream, chartSet));
		}
		CharArrayBuffer buffer = new CharArrayBuffer(i);
		try {
			char[] tmp = new char[1024];
			int l;
			while ((l = reader.read(tmp)) != -1) {
				buffer.append(tmp, 0, l);
			}
		} finally {
			reader.close();
		}

		return buffer.toString();
	}

	/**
	 * @deprecated use entityToString()
	 * @param in
	 * @return
	 * @throws ResponseException
	 */
	@SuppressWarnings("unused")
	private String inputStreamToString(final InputStream in) throws IOException {
		if (null == in) {
			return null;
		}

		BufferedReader reader = new BufferedReader(new InputStreamReader(in,
				"UTF-8"));
		StringBuffer buf = new StringBuffer();
		try {
			char[] buffer = new char[1024];
			while ((reader.read(buffer)) != -1) {
				buf.append(buffer);
			}
			return buf.toString();
		} finally {
			if (reader != null) {
				reader.close();
				setStreamConsumed(true);
			}
		}
	}

//	public JSONObject asJSONObject() throws ResponseException {
//		try {
//			return new JSONObject(asString());
//		} catch (JSONException jsone) {
//			throw new ResponseException(jsone.getMessage() + ":" + asString(),
//					jsone);
//		}
//	}
//
//	public JSONArray asJSONArray() throws ResponseException {
//		try {
//			return new JSONArray(asString());
//		} catch (Exception jsone) {
//			throw new ResponseException(jsone.getMessage(), jsone);
//		}
//	}

	private void setStreamConsumed(boolean mStreamConsumed) {
		this.mStreamConsumed = mStreamConsumed;
	}

	public boolean isStreamConsumed() {
		return mStreamConsumed;
	}

	/**
	 * @deprecated
	 * @return
	 */
	public Document asDocument() {
		// TODO Auto-generated method stub
		return null;
	}

	private static Pattern escaped = Pattern.compile("&#([0-9]{3,5});");

	/**
	 * Unescape UTF-8 escaped characters to string.
	 * 
	 * @author pengjianq...@gmail.com
	 * 
	 * @param original
	 *            The string to be unescaped.
	 * @return The unescaped string
	 */
	public static String unescape(String original) {
		Matcher mm = escaped.matcher(original);
		StringBuffer unescaped = new StringBuffer();
		while (mm.find()) {
			mm.appendReplacement(unescaped, Character.toString((char) Integer
					.parseInt(mm.group(1), 10)));
		}
		mm.appendTail(unescaped);
		return unescaped.toString();
	}
	
	public long getContentLength(){
		return this.mResponse.getEntity().getContentLength();
	}
	
	public HttpResponse getHttpResponse(){
		return this.mResponse;
	}

	/**
	 * @return the mAccept_Ranges
	 */
	public String getmAccept_Ranges() {
		return mAccept_Ranges;
	}

	/**
	 * @param mAccept_Ranges the mAccept_Ranges to set
	 */
	public void setmAccept_Ranges(String mAccept_Ranges) {
		this.mAccept_Ranges = mAccept_Ranges;
	}

	/**
	 * @return the mContent_Range
	 */
	public String getmContent_Range() {
		return mContent_Range;
	}

	/**
	 * @param mContent_Range the mContent_Range to set
	 */
	public void setmContent_Range(String mContent_Range) {
		this.mContent_Range = mContent_Range;
	}

	
	
	
}
